#ifndef __XtideScanNav_h_included
// Definition of interface dialog between ScanNav and XTide
// Using WM_COPYDATA message


// Version of protocol
#define SCNNVXTIDE_VERSION 3


//Different possible values of dwData field
//=========================================
#define XTIDETALKER 0x8000
#define SCNNVTALKER 0x4000

// Xtide => ScanNav
#define XTIDE_REGISTERINFO XTIDETALKER|1
#define XTIDE_TIDEINFO XTIDETALKER|2
#define XTIDE_CLICKINFO XTIDETALKER|3

// ScanNav => XTide
#define SCNNV_REGISTERREQUEST SCNNVTALKER|1
#define SCNNV_TIDESREQUEST SCNNVTALKER|2
#define SCNNV_JUMPTOLOCATION SCNNVTALKER|3
#define SCNNV_HLTIDESREQUEST SCNNVTALKER|4


//================================================
// parameters to the SCNNV_TIDESREQUEST sent from ScanNav to Xtide
struct scnnv_TideRequest
{
  time_t requestedtime; // Time at requested time 
  int nstations;
  struct {
    float lat;
    float lon;
  } stations[1]; // One entry per requested station
};

//================================================
// parameters to the SCNNV_HLTIDESREQUEST sent from ScanNav to Xtide
struct scnnv_HLTideRequest
{
  time_t requestedtime; // Time at requested time 
  int nstations;
  struct {
    BOOL wantslowtide;
    float lat;
    float lon;
  } stations[1]; // One entry per requested station
};

//================================================
// Tides information sent from XTide to ScanNav 
// Sent with ident XTIDE_TIDEINFO in reply to SCNNV_TIDESREQUEST

// One entry of _SationTideInfo per requested station
struct _SationTideInfo
{
  float lat; // latitude of station
  float lon; // latitude of station
  time_t tidetime; // Time of nearset tide
  float ht_height; // Height at nearest tide time
  float req_height; // height at requested time
  time_t secondtidetime; // Time of other nearest heigh tide
};
struct xtide_TideInfo
{
  time_t requestedtime; // Time at requested time 
  int coef; // Coefficiant of tide
  int nstations; // number of elements in stations[] array
  struct _SationTideInfo stations[1];
};

//================================================
// date information sent from XTide to ScanNav (using XTIDE_CLICKINFO) when user clicks on curve 
struct xtide_ClickInfo
{
  time_t requestedtime; // Time at requested time 
};


// Description of sequence of messages:
// ScanNav First searchs for the WXTide window using "FindWindow"
// and sends him a SCNNV_REGISTERREQUEST message to let him now ScanNav is listening
// ScanNav then sends messages to request tide info,
// and WXTide sends tide info either on request or when user interacts on its windows
// ScanNav closes the connection by sending a SCNNV_REGISTERREQUEST with a NULL HWND

// 1) Messages initiated From ScanNav to XTide:
//=============================================

// dwData:SCNNV_REGISTERREQUEST
// lpData: HWND of ScanNav control dialog
// Sent by ScanNav to WXtide to register the control dialog window (HWND!=NULL)
// or to unregister (HWND==NULL)
// If HWND is not NULL, reply of WXTide with:
//   dwData: XTIDE_REGISTERINFO
//   lpData: version of protocol (int)=> Should be SCNNVXTIDE_VERSION
// No reply if HWND is NULL

// dwData:SCNNV_TIDESREQUEST
// lpData: struct scnnv_TideRequest with requested time, and station(s) position(s) list
// Sent by ScanNav to WXtide to request tides information
// reply with:
//   dwData: XTIDE_TIDEINFO
//   lpData: struct xtide_TideInfo with infos on all requested stations

// dwData:SCNNV_JUMPTOLOCATION
// lpData: float[2]: latitude/longitude of requested location
// Sent by ScanNav to WXtide to position XTide to a specified station 
// No requested reply of WXTide.


// 2)  Messages initiated From XTide to ScanNav:
//=============================================

// dwData:XTIDE_CLICKINFO
// lpData: struct xtide_ClickInfo containing date corresponding to the clicked point
// Sent by WXtide to WXTide when user clicks on the curve so as to notify ScanNav
// ScanNav will then query for tides corresponding to this date using SCNNV_TIDESREQUEST (not mandatory)


#endif //__XtideScanNav_h_included
